!include LogicLib.nsh
!include WinMessages.nsh
!include FileFunc.nsh
!insertmacro GetDrives
!insertmacro DriveSpace

Name "LiveCPD2 ${VERSION}"
OutFile "livecpd2.exe"
; InstallDir $PROGRAMFILES\livecpd2
RequestExecutionLevel admin
AllowRootDirInstall true

Var target_dir
ReserveFile syslinux.exe
ReserveFile makebootfat.exe
ReserveFile ldlinux.bss
ReserveFile ldlinux.sys
ReserveFile mbrfat.bin
;ReserveFile syslinux.cfg
;ReserveFile bootmsg.txt
;ReserveFile initrd.gz
;ReserveFile bzImage
;ReserveFile root.img
Function install_dir
    SetOutPath $target_dir
    File /oname=$OUTDIR\syslinux.cfg syslinux.cfg
    File /oname=$OUTDIR\bootmsg.txt bootmsg.txt
    File /oname=$OUTDIR\initrd.gz initrd.gz
    File /oname=$OUTDIR\vmlinuz bzImage
    File /oname=$OUTDIR\root.img root.img
    IfFileExists $OUTDIR\override.img NoOverride CreateOverride
    CreateOverride:
    MessageBox MB_YESNO|MB_ICONQUESTION "Do you want to use the USB device to store override configuration information?" IDNO NoOverride
    File /oname=$OUTDIR\override.img override.img
    NoOverride:
FunctionEnd

Var install_mode
!define MODE_MAKEBOOTFAT_AUTO   "USB-ZIP format auto detection (recomended)"
!define MODE_MAKEBOOTFAT_MANUAL "USB-ZIP format manual selection"
!define MODE_SYSLINUX           "USB-HDD addition (non-destructive)"

Var target_drive
Function run_syslinux
    MessageBox MB_YESNO|MB_ICONEXCLAMATION "Are you sure you want to install to $target_drive:\ ?$\n$\nThis will overwrite the boot sector of that drive rendering it unable to boot anything except LiveCPD2." IDNO NoSysLinuxInstall

    StrCpy $target_dir "$target_drive:"
    File /oname=$TEMP\syslinux.exe syslinux.exe
    Call install_dir
    ExecWait '"$TEMP\syslinux.exe" $target_drive:' $0
    ${If} $0 != 0
        MessageBox MB_ICONEXCLAMATION "Boot sector write failed (syslinux.exe returned a non-zero exit code)."
        Abort
    ${EndIf}
    Delete $TEMP\syslinux.exe

    return
    NoSysLinuxInstall:
    Abort
FunctionEnd

Function run_makebootfat
    ${If} $install_mode == "${MODE_MAKEBOOTFAT_AUTO}"
        MessageBox MB_YESNO|MB_ICONEXCLAMATION "Are you sure you want to install to the auto-detected USB device?  Ensure exactly one USB drive is connected.$\n$\nThis will ERASE ALL DATA on that drive." IDNO NoMakeBootFatInstall
        StrCpy $1 "usb"
    ${Else}
        MessageBox MB_YESNO|MB_ICONEXCLAMATION "Are you sure you want to install to $target_drive  ?$\n$\nThis will ERASE ALL DATA on that drive." IDNO NoMakeBootFatInstall
        StrCpy $1 "\\.\$target_drive"
    ${EndIf}

    File /oname=$TEMP\makebootfat.exe makebootfat.exe
    File /oname=$TEMP\ldlinux.bss ldlinux.bss
    File /oname=$TEMP\ldlinux.sys ldlinux.sys
    File /oname=$TEMP\mbrfat.bin mbrfat.bin
    StrCpy $target_dir "$TEMP\lcpd2img"
    CreateDirectory $target_dir
    Call install_dir

    DetailPrint "Rebuilding device image, this may take a while."
    ExecWait '"$TEMP\makebootfat.exe" -o $1 -Y -Z -b $TEMP\ldlinux.bss -m $TEMP\mbrfat.bin -F -c $TEMP\ldlinux.sys $target_dir' $0
    ${If} $0 != 0
        MessageBox MB_ICONEXCLAMATION "Rebuilding device image failed (makebootfat.exe returned a non-zero status code)."
        Abort
    ${EndIf}
    Delete $TEMP\makebootfat.exe
    Delete $TEMP\ldlinux.bss
    Delete $TEMP\ldlinux.sys
    Delete $TEMP\mbrfat.bin
    RMDir /r $TEMP\lcpd2img

    return
    NoMakeBootFatInstall:
    Abort
FunctionEnd


Page Custom ModeSelectCreate ModeSelectLeave 
Page Custom DriveSelectCreate DriveSelectLeave 
Page Instfiles

Section "" 
    ${If} $install_mode == "${MODE_MAKEBOOTFAT_AUTO}"
        Call run_makebootfat
    ${ElseIf} $install_mode == "${MODE_MAKEBOOTFAT_MANUAL}"
        Call run_makebootfat
    ${Else}
        Call run_syslinux
    ${EndIf}
SectionEnd

Function ModeSelectCreate 
    WriteIniStr '$PLUGINSDIR\mseldlg.ini' 'Settings' 'NumFields' '2'

    WriteIniStr '$PLUGINSDIR\mseldlg.ini' 'Field 1' 'Type' 'Label'
    WriteIniStr '$PLUGINSDIR\mseldlg.ini' 'Field 1' 'Left' '5'
    WriteIniStr '$PLUGINSDIR\mseldlg.ini' 'Field 1' 'Top' '5'
    WriteIniStr '$PLUGINSDIR\mseldlg.ini' 'Field 1' 'Right' '-6'
    WriteIniStr '$PLUGINSDIR\mseldlg.ini' 'Field 1' 'Bottom' '17'
    WriteIniStr '$PLUGINSDIR\mseldlg.ini' 'Field 1' 'Text' \
    'Select Installation type:'

    WriteIniStr '$PLUGINSDIR\mseldlg.ini' 'Field 2' 'Type' 'DropList'
    WriteIniStr '$PLUGINSDIR\mseldlg.ini' 'Field 2' 'Left' '30'
    WriteIniStr '$PLUGINSDIR\mseldlg.ini' 'Field 2' 'Top' '26'
    WriteIniStr '$PLUGINSDIR\mseldlg.ini' 'Field 2' 'Right' '-31'
    WriteIniStr '$PLUGINSDIR\mseldlg.ini' 'Field 2' 'Bottom' '100'
#    WriteIniStr '$PLUGINSDIR\mseldlg.ini' 'Field 2' 'Flags' 'Notify'
    

    Push $R0
    Push $R1
    ClearErrors
    ReadRegStr $R0 HKLM \
    "SOFTWARE\Microsoft\Windows NT\CurrentVersion" CurrentVersion
    IfErrors 0 lbl_winnt
    ; we are not NT
    ReadRegStr $R0 HKLM \
    "SOFTWARE\Microsoft\Windows\CurrentVersion" VersionNumber
    StrCpy $R1 $R0 1
    StrCmp $R1 '4' 0 lbl_done
    StrCpy $R1 $R0 3
    StrCmp $R1 '4.0' lbl_win32_95
    StrCmp $R1 '4.9' lbl_win32_ME lbl_win32_98

    lbl_winnt:
    StrCpy $R1 $R0 1
    StrCmp $R1 '3' lbl_winnt_x
    StrCmp $R1 '4' lbl_winnt_x
    StrCpy $R1 $R0 3
    StrCmp $R1 '5.0' lbl_winnt_2000
    StrCmp $R1 '5.1' lbl_winnt_XP
    StrCmp $R1 '5.2' lbl_winnt_2003
    StrCmp $R1 '6.0' lbl_winnt_vista
    StrCmp $R1 '6.1' lbl_winnt_7
    Goto lbl_winnt_7


    lbl_win32_95:
    lbl_win32_98:
    lbl_win32_ME:
    lbl_winnt_x:
    lbl_winnt_2000:
    lbl_winnt_XP:
    lbl_winnt_2003:

    WriteIniStr '$PLUGINSDIR\mseldlg.ini' 'Field 2' 'State' '${MODE_MAKEBOOTFAT_AUTO}'
    WriteIniStr '$PLUGINSDIR\mseldlg.ini' 'Field 2' 'ListItems' '${MODE_MAKEBOOTFAT_AUTO}|${MODE_MAKEBOOTFAT_MANUAL}|${MODE_SYSLINUX}'

    Goto lbl_done

    lbl_winnt_vista:
    lbl_winnt_7:

    WriteIniStr '$PLUGINSDIR\mseldlg.ini' 'Field 2' 'State' '${MODE_SYSLINUX}'
    WriteIniStr '$PLUGINSDIR\mseldlg.ini' 'Field 2' 'ListItems' '${MODE_SYSLINUX}'
 
    Goto lbl_done

lbl_done:
    Pop $R1
    Exch $R0


    push $0
    InstallOptions::Dialog '$PLUGINSDIR\mseldlg.ini'
    pop $0
    pop $0

FunctionEnd

Function ModeSelectLeave
    ReadIniStr $0 '$PLUGINSDIR\mseldlg.ini' 'Field 2' 'State'
    StrCpy $install_mode $0
FunctionEnd

Function DriveSelectCreate
    ${If} $install_mode == "${MODE_MAKEBOOTFAT_AUTO}"
        return
    ${EndIf}

         WriteIniStr '$PLUGINSDIR\gdcstm.ini' 'Settings' 'NumFields' '6'
 
         WriteIniStr '$PLUGINSDIR\gdcstm.ini' 'Field 1' 'Type' 'Label'
         WriteIniStr '$PLUGINSDIR\gdcstm.ini' 'Field 1' 'Left' '5'
         WriteIniStr '$PLUGINSDIR\gdcstm.ini' 'Field 1' 'Top' '5'
         WriteIniStr '$PLUGINSDIR\gdcstm.ini' 'Field 1' 'Right' '-6'
         WriteIniStr '$PLUGINSDIR\gdcstm.ini' 'Field 1' 'Bottom' '17'
         WriteIniStr '$PLUGINSDIR\gdcstm.ini' 'Field 1' 'Text' \
         'Select Installation drive:'
 
         StrCpy $R2 0
         StrCpy $R0 ''
         ${GetDrives} "FDD" GetDrivesCallBack
 
         WriteIniStr '$PLUGINSDIR\gdcstm.ini' 'Field 2' 'Type' 'DropList'
         WriteIniStr '$PLUGINSDIR\gdcstm.ini' 'Field 2' 'Left' '30'
         WriteIniStr '$PLUGINSDIR\gdcstm.ini' 'Field 2' 'Top' '26'
         WriteIniStr '$PLUGINSDIR\gdcstm.ini' 'Field 2' 'Right' '-31'
         WriteIniStr '$PLUGINSDIR\gdcstm.ini' 'Field 2' 'Bottom' '100'
         WriteIniStr '$PLUGINSDIR\gdcstm.ini' 'Field 2' 'Flags' 'Notify'
         WriteIniStr '$PLUGINSDIR\gdcstm.ini' 'Field 2' 'State' '$R1'
         WriteIniStr '$PLUGINSDIR\gdcstm.ini' 'Field 2' 'ListItems' '$R0'
 
         WriteIniStr '$PLUGINSDIR\gdcstm.ini' 'Field 3' 'Type' 'Label'
         WriteIniStr '$PLUGINSDIR\gdcstm.ini' 'Field 3' 'Left' '5'
         WriteIniStr '$PLUGINSDIR\gdcstm.ini' 'Field 3' 'Top' '109'
         WriteIniStr '$PLUGINSDIR\gdcstm.ini' 'Field 3' 'Right' '59'
         WriteIniStr '$PLUGINSDIR\gdcstm.ini' 'Field 3' 'Bottom' '119'
         WriteIniStr '$PLUGINSDIR\gdcstm.ini' 'Field 3' 'Text' \
         'Space required:'
 
         WriteIniStr '$PLUGINSDIR\gdcstm.ini' 'Field 4' 'Type' 'Label'
         WriteIniStr '$PLUGINSDIR\gdcstm.ini' 'Field 4' 'Left' '60'
         WriteIniStr '$PLUGINSDIR\gdcstm.ini' 'Field 4' 'Top' '109'
         WriteIniStr '$PLUGINSDIR\gdcstm.ini' 'Field 4' 'Right' '-5'
         WriteIniStr '$PLUGINSDIR\gdcstm.ini' 'Field 4' 'Bottom' '119'
         WriteIniStr '$PLUGINSDIR\gdcstm.ini' 'Field 4' 'Text' \
         '${SIZE} Mb'
 
         WriteIniStr '$PLUGINSDIR\gdcstm.ini' 'Field 5' 'Type' 'Label'
         WriteIniStr '$PLUGINSDIR\gdcstm.ini' 'Field 5' 'Left' '5'
         WriteIniStr '$PLUGINSDIR\gdcstm.ini' 'Field 5' 'Top' '120'
         WriteIniStr '$PLUGINSDIR\gdcstm.ini' 'Field 5' 'Right' '59'
         WriteIniStr '$PLUGINSDIR\gdcstm.ini' 'Field 5' 'Bottom' '130'
         WriteIniStr '$PLUGINSDIR\gdcstm.ini' 'Field 5' 'Text' \
         'Space available:'
 
         WriteIniStr '$PLUGINSDIR\gdcstm.ini' 'Field 6' 'Type' 'Label'
         WriteIniStr '$PLUGINSDIR\gdcstm.ini' 'Field 6' 'Left' '60'
         WriteIniStr '$PLUGINSDIR\gdcstm.ini' 'Field 6' 'Top' '120'
         WriteIniStr '$PLUGINSDIR\gdcstm.ini' 'Field 6' 'Right' '-5'
         WriteIniStr '$PLUGINSDIR\gdcstm.ini' 'Field 6' 'Bottom' '130'
         WriteIniStr '$PLUGINSDIR\gdcstm.ini' 'Field 6' 'Text' \
         '$R3 Mb'
 
         push $0
         InstallOptions::Dialog '$PLUGINSDIR\gdcstm.ini'
         pop $0
         pop $0
FunctionEnd
 
Function DriveSelectLeave
        ReadIniStr $0 '$PLUGINSDIR\gdcstm.ini' 'Settings' 'State'
        StrCmp $0 '2' 0 next
        ReadIniStr $0 '$PLUGINSDIR\gdcstm.ini' 'Field 2' 'State'
        StrCpy $0 $0 3
        ${DriveSpace} "$0" "/D=T /S=M" $R3
        WriteIniStr '$PLUGINSDIR\gdcstm.ini' 'Field 6' 'Text' \
        '$R3 Mb'
        ReadIniStr $0 '$PLUGINSDIR\gdcstm.ini' 'Field 6' 'HWND'
        SendMessage $0 ${WM_SETTEXT} 0 'STR:$R3 Mb'
        Abort
 
     next:
        ReadIniStr $0 '$PLUGINSDIR\gdcstm.ini' 'Field 2' 'State'
        StrCpy $target_drive $0 1
FunctionEnd
 
Function GetDrivesCallBack
        ${DriveSpace} "$9" "/D=T /S=M" $R4
        IntCmp $R4 '${SIZE}' end end def
        StrCmp $9 "C:\" end
    def:
        StrCmp $R2 '0' 0 next
        StrCpy $R3 '$R4'
        StrCpy $R1 '$9'
        IntOp $R2 $R2 + 1
    next:
        StrCpy $R0 '$R0$9|'
    end:
    Push $0
FunctionEnd
 
Function .onInit
    InitPluginsDir
    GetTempFileName $0
    Rename $0 '$PLUGINSDIR\gdcstm.ini'
    GetTempFileName $0
    Rename $0 '$PLUGINSDIR\mseldlg.ini'
FunctionEnd


Function .onInstFailed
    Delete $TEMP\syslinux.exe
    Delete $TEMP\makebootfat.exe
    Delete $TEMP\ldlinux.bss
    Delete $TEMP\ldlinux.sys
    Delete $TEMP\mbrfat.bin
    Delete '$PLUGINSDIR\gdcstm.ini'
    Delete '$PLUGINSDIR\mseldlg.ini'
    RMDir /r $TEMP\lcpd2img
FunctionEnd

Function .onInstSuccess
    Delete '$PLUGINSDIR\gdcstm.ini'
    Delete '$PLUGINSDIR\mseldlg.ini'
FunctionEnd
