#include <linux/unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/mount.h>
#include <errno.h>
#include <string.h>
#include <time.h>

#include "startup.h"
#include "filesys.h"
#include "hotplug.h"
#include "probedev.h"
#include "setuproot.h"

#define MAX_WAIT_TIME	    20
#define HOTPLUG_REWAITTIME  5

char *g_root_path = NULL;
int g_root_major = 0;
int g_root_minor = 0;
int g_root_setup = 0;

char *g_override_path = NULL;
int g_override_major = 0;
int g_override_minor = 0;
int g_override_setup = 0;

/* Not defined correctly in headers */
int pivot_root(const char *new_root, const char *put_old);

int main(int argc, char **argv) {
	time_t endwait, timenow;
	char *envp[] = { NULL };
    int i;
	/* If called as hotplug, do that */
	if (argc > 0 && argv && argv[0]) {
		if (strstr(argv[0], "hotplug")) {
			exec_hotplug(argc, argv);
			return 0;
		}
	}

	/* Otherwise, fixup to be init */
	if (!argv || argc < 1) argv=(char **)malloc(sizeof(char *)*2);
	argv[0] = (char *)malloc(5);
	strcpy(argv[0], "init");
	
	/* Setup console, etc */
	init_startup();

	/* Mount basic filesystems */
	init_filesystem();

	/* Setup hotplug pipes */
	init_hotplug();

    /* Clear the screen: 2.6.31.4 has/had resets the cursor but doesn't clear 
       the screen, resulting in garbled output */
    for (i=0; i<48; i++) { printf("\n"); }

	/* Probe everything already initialized */
	probe_all_devices();

	endwait = time(NULL) + MAX_WAIT_TIME;	
	while (1) {
		if (!g_root_setup && g_root_path != NULL) {
			setup_root_filesystem(g_root_major, g_root_minor, g_root_path);
			g_root_setup = 1;
    		if (hotplug_process()) {
                timenow = time(NULL);
                if (timenow+HOTPLUG_REWAITTIME > endwait)
    	    		endwait = timenow + HOTPLUG_REWAITTIME;
            }
		}

		if (g_root_setup && !g_override_setup && g_override_path != NULL) {
			extract_override(g_override_major, g_override_minor, g_override_path);
			g_override_setup = 1;
		}

		timenow = time(NULL);

		if (g_root_setup && (g_override_setup || timenow > endwait)) {
			break;
		}
				
		sleep(1);
        if (hotplug_process()) {
            if (timenow+HOTPLUG_REWAITTIME > endwait)
                endwait = timenow + HOTPLUG_REWAITTIME;
        }
	}

	/* Cleanup and switch the new real root */
	cleanup_hotplug();
	cleanup_filesystem();

	/* Switch to new root */
	if (mkdir(ROOT_LOC "/initrd", 0700) != 0) {
		fprintf(stderr, "Unable to create temporary root relocation: %s\n", strerror(errno));
		exit(1);
	}
	if (pivot_root(ROOT_LOC, ROOT_LOC "/initrd") != 0) {
		fprintf(stderr, "Unable to pivot root: %s\n", strerror(errno));
		exit(1);
	}
	/* We are now "in" the real root */
	chdir("/");

	/* Unmount and destroy initrd */
	/*if (umount("/initrd") == 0) {
		unlink("/initrd");
	} else {
		fprintf(stderr, "Warning: unable to unmount initrd: %s\n", strerror(errno));
	}*/

    printf("InitRD setup done, starting init...\n\n"); fflush(stdout);

    /* Close terminal usage */
    close(1);
    close(2);

	/* Start real init */
	execve("/sbin/init", argv, envp);

    /* Can't dump an error here, unfortunaly */
    exit(1);
	return 1;
}
